@@ -3,8 +3,13 @@ import sys
 import builtins
 
 from .socket import send
-from .generated.inputs import read
-from .generated.outputs import display, display_plotly
+from .page import Page
+
+try:
+    from .generated.inputs import read
+    from .generated.outputs import display_plotly
+except ImportError:
+    pass
 
 
 def writeWraper(type, write, text):
@@ -28,7 +33,13 @@ def _overload_input():
 
 
 def _overload_print():
-    builtins.print = display
+    def print_overload(*values):
+        page = Page()
+        for value in values:
+            page.display(value)
+        return page.run()
+
+    builtins.print = print_overload
 
 
 def _overload_plotly_show():
